@@ -1,4 +1,4 @@
-# Copyright 2012-2018, Damian Johnson and The Tor Project
+# Copyright 2012-2019, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -8,6 +8,7 @@ Connection and networking based utility functions.
 
 ::
 
+  download - download from a given url
   get_connections - quieries the connections belonging to a given process
   system_resolvers - provides connection resolution methods that are likely to be available
   port_usage - brief description of the common usage for a port
@@ -47,7 +48,7 @@ Connection and networking based utility functions.
   **NETSTAT_WINDOWS**   netstat command under Windows
   **SS**                ss command
   **LSOF**              lsof command
-  **SOCKSTAT**          sockstat command under \*nix
+  **SOCKSTAT**          sockstat command under \\*nix
   **BSD_SOCKSTAT**      sockstat command under FreeBSD
   **BSD_PROCSTAT**      procstat command under FreeBSD
   **BSD_FSTAT**         fstat command under OpenBSD
@@ -55,18 +56,26 @@ Connection and networking based utility functions.
 """
 
 import collections
-import hashlib
-import hmac
 import os
 import platform
 import re
+import socket
+import sys
+import time
 
+import stem
 import stem.util
 import stem.util.proc
 import stem.util.system
 
 from stem.util import conf, enum, log, str_tools
 
+try:
+  # account for urllib's change between python 2.x and 3.x
+  import urllib.request as urllib
+except ImportError:
+  import urllib2 as urllib
+
 # Connection resolution is risky to log about since it's highly likely to
 # contain sensitive information. That said, it's also difficult to get right in
 # a platform independent fashion. To opt into the logging requried to
@@ -89,8 +98,6 @@ Resolver = enum.Enum(
 FULL_IPv4_MASK = '255.255.255.255'
 FULL_IPv6_MASK = 'FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF:FFFF'
 
-CRYPTOVARIABLE_EQUALITY_COMPARISON_NONCE = os.urandom(32)
-
 PORT_USES = None  # port number => description
 
 RESOLVER_COMMAND = {
@@ -125,28 +132,28 @@ RESOLVER_FILTER = {
   Resolver.PROC: '',
 
   # tcp        0    586 192.168.0.1:44284       38.229.79.2:443         ESTABLISHED 15843/tor
-  Resolver.NETSTAT: '^{protocol}\s+.*\s+{local}\s+{remote}\s+ESTABLISHED\s+{pid}/{name}\s*$',
+  Resolver.NETSTAT: '^{protocol}\\s+.*\\s+{local}\\s+{remote}\\s+ESTABLISHED\\s+{pid}/{name}\\s*$',
 
   # tcp        586 192.168.0.1:44284       38.229.79.2:443         ESTABLISHED 15843
-  Resolver.NETSTAT_WINDOWS: '^\s*{protocol}\s+{local}\s+{remote}\s+ESTABLISHED\s+{pid}\s*$',
+  Resolver.NETSTAT_WINDOWS: '^\\s*{protocol}\\s+{local}\\s+{remote}\\s+ESTABLISHED\\s+{pid}\\s*$',
 
   # tcp    ESTAB      0      0           192.168.0.20:44415       38.229.79.2:443    users:(("tor",15843,9))
-  Resolver.SS: '^{protocol}\s+ESTAB\s+.*\s+{local}\s+{remote}\s+users:\(\("{name}",(?:pid=)?{pid},(?:fd=)?[0-9]+\)\)$',
+  Resolver.SS: '^{protocol}\\s+ESTAB\\s+.*\\s+{local}\\s+{remote}\\s+users:\\(\\("{name}",(?:pid=)?{pid},(?:fd=)?[0-9]+\\)\\)$',
 
   # tor  3873  atagar  45u  IPv4  40994  0t0  TCP 10.243.55.20:45724->194.154.227.109:9001 (ESTABLISHED)
-  Resolver.LSOF: '^{name}\s+{pid}\s+.*\s+{protocol}\s+{local}->{remote} \(ESTABLISHED\)$',
+  Resolver.LSOF: '^{name}\\s+{pid}\\s+.*\\s+{protocol}\\s+{local}->{remote} \\(ESTABLISHED\\)$',
 
   # atagar   tor                  15843    tcp4   192.168.0.20:44092        68.169.35.102:443         ESTABLISHED
-  Resolver.SOCKSTAT: '^\S+\s+{name}\s+{pid}\s+{protocol}4\s+{local}\s+{remote}\s+ESTABLISHED$',
+  Resolver.SOCKSTAT: '^\\S+\\s+{name}\\s+{pid}\\s+{protocol}4\\s+{local}\\s+{remote}\\s+ESTABLISHED$',
 
   # _tor     tor        4397  12 tcp4   172.27.72.202:54011   127.0.0.1:9001
-  Resolver.BSD_SOCKSTAT: '^\S+\s+{name}\s+{pid}\s+\S+\s+{protocol}4\s+{local}\s+{remote}$',
+  Resolver.BSD_SOCKSTAT: '^\\S+\\s+{name}\\s+{pid}\\s+\\S+\\s+{protocol}4\\s+{local}\\s+{remote}$',
 
   # 3561 tor                 4 s - rw---n--   2       0 TCP 10.0.0.2:9050 10.0.0.1:22370
-  Resolver.BSD_PROCSTAT: '^\s*{pid}\s+{name}\s+.*\s+{protocol}\s+{local}\s+{remote}$',
+  Resolver.BSD_PROCSTAT: '^\\s*{pid}\\s+{name}\\s+.*\\s+{protocol}\\s+{local}\\s+{remote}$',
 
   # _tor     tor        15843   20* internet stream tcp 0x0 192.168.1.100:36174 --> 4.3.2.1:443
-  Resolver.BSD_FSTAT: '^\S+\s+{name}\s+{pid}\s+.*\s+{protocol}\s+\S+\s+{local}\s+[-<]-[->]\s+{remote}$',
+  Resolver.BSD_FSTAT: '^\\S+\\s+{name}\\s+{pid}\\s+.*\\s+{protocol}\\s+\\S+\\s+{local}\\s+[-<]-[->]\\s+{remote}$',
 }
 
 
@@ -166,6 +173,47 @@ class Connection(collections.namedtuple('Connection', ['local_address', 'local_p
   """
 
 
+def download(url, timeout = None, retries = None):
+  """
+  Download from the given url.
+
+  .. versionadded:: 1.8.0
+
+  :param str url: uncompressed url to download from
+  :param int timeout: timeout when connection becomes idle, no timeout applied
+    if **None**
+  :param int retires: maximum attempts to impose
+
+  :returns: **bytes** content of the given url
+
+  :raises:
+    * :class:`~stem.DownloadTimeout` if our request timed out
+    * :class:`~stem.DownloadFailed` if our request fails
+  """
+
+  if retries is None:
+    retries = 0
+
+  start_time = time.time()
+
+  try:
+    return urllib.urlopen(url, timeout = timeout).read()
+  except socket.timeout as exc:
+    raise stem.DownloadTimeout(url, exc, sys.exc_info()[2], timeout)
+  except:
+    exc, stacktrace = sys.exc_info()[1:3]
+
+    if timeout is not None:
+      timeout -= time.time() - start_time
+
+    if retries > 0 and (timeout is None or timeout > 0):
+      log.debug('Failed to download from %s (%i retries remaining): %s' % (url, retries, exc))
+      return download(url, timeout, retries - 1)
+    else:
+      log.debug('Failed to download from %s: %s' % (url, exc))
+      raise stem.DownloadFailed(url, exc, stacktrace)
+
+
 def get_connections(resolver = None, process_pid = None, process_name = None):
   """
   Retrieves a list of the current connections for a given process. This
@@ -243,11 +291,11 @@ def get_connections(resolver = None, process_pid = None, process_name = None):
     raise IOError("Unable to query '%s': %s" % (resolver_command, exc))
 
   resolver_regex_str = RESOLVER_FILTER[resolver].format(
-    protocol = '(?P<protocol>\S+)',
-    local = '(?P<local>[\[\]0-9a-f.:]+)',
-    remote = '(?P<remote>[\[\]0-9a-f.:]+)',
+    protocol = '(?P<protocol>\\S+)',
+    local = '(?P<local>[\\[\\]0-9a-f.:]+)',
+    remote = '(?P<remote>[\\[\\]0-9a-f.:]+)',
     pid = process_pid if process_pid else '[0-9]*',
-    name = process_name if process_name else '\S*',
+    name = process_name if process_name else '\\S*',
   )
 
   _log('Resolver regex: %s' % resolver_regex_str)
@@ -721,6 +769,9 @@ def _get_binary(value, bits):
   return ''.join([str((value >> y) & 1) for y in range(bits - 1, -1, -1)])
 
 
+# TODO: In stem 2.x we should consider unifying this with
+# stem.client.datatype's _unpack_ipv4_address() and _unpack_ipv6_address().
+
 def _address_to_binary(address):
   """
   Provides the binary value for an IPv4 or IPv6 address.
@@ -739,34 +790,6 @@ def _address_to_binary(address):
     raise ValueError("'%s' is neither an IPv4 or IPv6 address" % address)
 
 
-def _hmac_sha256(key, msg):
-  """
-  Generates a sha256 digest using the given key and message.
-
-  :param str key: starting key for the hash
-  :param str msg: message to be hashed
-
-  :returns: sha256 digest of msg as bytes, hashed using the given key
-  """
-
-  return hmac.new(key, msg, hashlib.sha256).digest()
-
-
-def _cryptovariables_equal(x, y):
-  """
-  Compares two strings for equality securely.
-
-  :param str x: string to be compared.
-  :param str y: the other string to be compared.
-
-  :returns: **True** if both strings are equal, **False** otherwise.
-  """
-
-  return (
-    _hmac_sha256(CRYPTOVARIABLE_EQUALITY_COMPARISON_NONCE, x) ==
-    _hmac_sha256(CRYPTOVARIABLE_EQUALITY_COMPARISON_NONCE, y))
-
-
 # TODO: drop with stem 2.x
 # We renamed our methods to drop a redundant 'get_*' prefix, so alias the old
 # names for backward compatability.
