@@ -1,4 +1,4 @@
-# Copyright 2015-2017, Damian Johnson and The Tor Project
+# Copyright 2015-2019, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -116,7 +116,7 @@ def skip(msg):
 
 def asynchronous(func):
   test = stem.util.test_tools.AsyncTest(func)
-  ASYNC_TESTS['%s.%s' % (func.__module__, func.__name__)] = test
+  ASYNC_TESTS[test.name] = test
   return test.method
 
 
@@ -143,12 +143,13 @@ class AsyncTest(object):
   """
 
   def __init__(self, runner, args = None, threaded = False):
+    self.name = '%s.%s' % (runner.__module__, runner.__name__)
+
     self._runner = runner
     self._runner_args = args
     self._threaded = threaded
 
     self.method = lambda test: self.result(test)  # method that can be mixed into TestCases
-    self.method.async = self
 
     self._process = None
     self._process_pipe = None
@@ -187,7 +188,13 @@ class AsyncTest(object):
         self._process_pipe, child_pipe = multiprocessing.Pipe()
 
         if self._threaded:
-          self._process = threading.Thread(target = _wrapper, args = (child_pipe, self._runner, self._runner_args))
+          self._process = threading.Thread(
+            target = _wrapper,
+            args = (child_pipe, self._runner, self._runner_args),
+            name = 'Background test of %s' % self.name,
+          )
+
+          self._process.setDaemon(True)
         else:
           self._process = multiprocessing.Process(target = _wrapper, args = (child_pipe, self._runner, self._runner_args))
 
@@ -267,6 +274,19 @@ class TimedTestRunner(unittest.TextTestRunner):
           else:
             return super(original_type, self).assertItemsEqual(expected, actual)
 
+        def assertRaisesWith(self, exc_type, exc_msg, func, *args, **kwargs):
+          """
+          Asserts the given invokation raises the expected excepiton. This is
+          similar to unittest's assertRaises and assertRaisesRegexp, but checks
+          for an exact match.
+
+          This method is **not** being vended to external users and may be
+          changed without notice. If you want this method to be part of our
+          vended API then please let us know.
+          """
+
+          return self.assertRaisesRegexp(exc_type, '^%s$' % re.escape(exc_msg), func, *args, **kwargs)
+
         def assertRaisesRegexp(self, exc_type, exc_msg, func, *args, **kwargs):
           if stem.prereq._is_python_26():
             try:
@@ -303,7 +323,7 @@ def test_runtimes():
 
 def clean_orphaned_pyc(paths):
   """
-  Deletes any file with a \*.pyc extention without a corresponding \*.py. This
+  Deletes any file with a \\*.pyc extention without a corresponding \\*.py. This
   helps to address a common gotcha when deleting python files...
 
   * You delete module 'foo.py' and run the tests to ensure that you haven't
@@ -435,6 +455,7 @@ def stylistic_issues(paths, check_newlines = False, check_exception_keyword = Fa
 
   ignore_rules = []
   ignore_for_file = []
+  ignore_all_for_files = []
 
   for rule in CONFIG['pycodestyle.ignore'] + CONFIG['pep8.ignore']:
     if '=>' in rule:
@@ -443,6 +464,8 @@ def stylistic_issues(paths, check_newlines = False, check_exception_keyword = Fa
       if ':' in rule_entry:
         rule, code = rule_entry.split(':', 1)
         ignore_for_file.append((path.strip(), rule.strip(), code.strip()))
+      elif rule_entry.strip() == '*':
+        ignore_all_for_files.append(path.strip())
     else:
       ignore_rules.append(rule)
 
@@ -451,6 +474,10 @@ def stylistic_issues(paths, check_newlines = False, check_exception_keyword = Fa
       if path.endswith(ignored_path) and ignored_rule == rule and code.strip().startswith(ignored_code):
         return True
 
+    for ignored_path in ignore_all_for_files:
+      if path.endswith(ignored_path):
+        return True
+
     return False
 
   if is_pycodestyle_available():
@@ -468,6 +495,10 @@ def stylistic_issues(paths, check_newlines = False, check_exception_keyword = Fa
 
         is_block_comment = False
 
+        for ignored_path in ignore_all_for_files:
+          if filename.endswith(ignored_path):
+            return
+
         for index, line in enumerate(lines):
           content = line.split('#', 1)[0].strip()
 
