@@ -6,9 +6,11 @@ import os
 import test
 
 __all__ = [
+  'client',
   'connection',
   'control',
   'descriptor',
+  'directory',
   'exit_policy',
   'socket',
   'util',
